@@ -444,7 +444,7 @@ else
             esac
     esac
     case $sbcl_arch in
-        arm64)
+        arm64|riscv)
             WITH_FEATURES="$WITH_FEATURES :sb-thread"
     esac
 fi
@@ -652,15 +652,8 @@ case "$sbcl_arch" in
 	$GNUMAKE -C tools-for-build where-is-mcontext -I ../src/runtime
 	tools-for-build/where-is-mcontext > src/runtime/ppc-linux-mcontext.h || (echo "error running where-is-mcontext"; exit 1)
     elif [ "$sbcl_os" = "darwin" ]; then
-        # We provide a dlopen shim, so a little lie won't hurt
-	printf ' :os-provides-dlopen' >> $ltf
-        # The default stack ulimit under darwin is too small to run PURIFY.
-        # Best we can do is complain and exit at this stage
-	if [ "`ulimit -s`" = "512" ]; then
-            echo "Your stack size limit is too small to build SBCL."
-            echo "See the limit(1) or ulimit(1) commands and the README file."
-            exit 1
-	fi
+	echo "Unsupported configuration"
+	exit 1
     fi
     ;;
   ppc64)
@@ -679,24 +672,6 @@ case "$sbcl_arch" in
         exit 1
     fi
     ;;
-  sparc)
-    # Test the compiler in order to see if we are building on Sun
-    # toolchain as opposed to GNU binutils, and write the appropriate
-    # FUNCDEF macro for assembler. No harm in running this on sparc-linux
-    # as well.
-    sh tools-for-build/sparc-funcdef.sh > src/runtime/sparc-funcdef.h
-    if [ "$sbcl_os" = "netbsd" ] || [ "$sbcl_os" = "sunos" ] || [ "$sbcl_os" = "linux" ]; then
-        printf ' :gencgc' >> $ltf
-    else
-        echo '***'
-        echo '*** You are running SPARC on other than SunOS, NetBSD, or Linux.  Since'
-        echo '*** GENCGC is untested on this combination, make-config.sh'
-        echo '*** is falling back to CHENEYGC.  Please consider adjusting'
-        echo '*** parms.lisp to build with GENCGC instead.'
-        echo '***'
-        printf ' :cheneygc' >> $ltf
-    fi
-    ;;
 esac
 
 # Use a little C program to try to guess the endianness.  Ware
