@@ -35,10 +35,17 @@
           ;; more-or-less confined to serve-event, except for a test which now
           ;; detects whether COMPUTE-POLLFDS is defined and therefore testable.
           :OS-PROVIDES-POLL
+          ;; Silly feature we add in because additive features are more sensible
+          ;; than subtractive ones. So you opt out, not in, to having *LOAD-TRUENAME*
+          ;; eagerly bound.
+          :ANSI-COMPLIANT-LOAD-TRUENAME
           ;; The final batch of symbols is strictly for C. The LISP_FEATURE_
           ;; prefix on the corresponding #define is unfortunate.
           :GCC-TLS :USE-SYS-MMAP
-          :OS-PROVIDES-BLKSIZE-T)")) ; only for 'src/runtime/wrap.h'
+          ;; only for 'src/runtime/wrap.h'
+          :OS-PROVIDES-BLKSIZE-T
+          ;; only for src/runtime/run-program.c
+          :OS-PROVIDES-CLOSE-RANGE-WRAPPER)"))
        (public-features
         (cons
          sb-impl::!sbcl-architecture
@@ -176,7 +183,7 @@
   result)
 
 ;;; Check for potentially bad format-control strings
-(defun !scan-format-control-strings ()
+(defun scan-format-control-strings ()
   (labels ((possibly-ungood-package-reference (string)
              ;; We want to see nothing SB-package-like at all
              (or (search "sb-" string :test #'char-equal)
@@ -211,12 +218,6 @@ Please check that all strings which were not recognizable to the compiler
 (as the first argument to WARN, etc.) are wrapped in SB-FORMAT:TOKENS"))
       wps)))
 
-(progn
-  ;; See the giant comment at the bottom of this file
-  ;; concerning the need for this GC.
-  (gc :full t)
-  (!scan-format-control-strings))
-
 ;;; Either set some more package docstrings, or remove any and all docstrings
 ;;; that snuck in (as can happen with any file compiled in warm load)
 ;;; depending on presence of the :sb-doc internal feature.
@@ -333,10 +334,12 @@ Please check that all strings which were not recognizable to the compiler
   ;; such that it now has neither properties nor globaldb info,
   ;; can have the slot set back to NIL if it wasn't already.
   (do-all-symbols (symbol)
-    (when (and (sb-kernel:symbol-info symbol)
-               (null (sb-kernel:symbol-info-vector symbol))
+    (when (and (sb-kernel:symbol-%info symbol) ; "raw" value is something
+               ;; but both "cooked" values are empty
+               (null (sb-kernel:symbol-dbinfo symbol))
                (null (symbol-plist symbol)))
-      (setf (sb-kernel:symbol-info symbol) nil)))
+      (sb-sys:%primitive sb-vm::set-slot symbol nil
+                         'make-symbol sb-vm:symbol-info-slot sb-vm:other-pointer-lowtag)))
 )
 
 (sb-ext:gc :full t)
@@ -401,7 +404,8 @@ Please check that all strings which were not recognizable to the compiler
    #+sb-devel
    (lambda (symbol accessibility)
      (declare (ignore accessibility))
-     (or (sb-kernel:symbol-info symbol)
+     (or (sb-kernel:symbol-%info symbol)
+         (sb-int:symbol-fdefn symbol) ; might be redundant with existence of %info, but ok
          (and (boundp symbol) (not (keywordp symbol)))))
    ;; Release mode: retain all symbols satisfying this intricate test
    #-sb-devel
@@ -454,7 +458,8 @@ Please check that all strings which were not recognizable to the compiler
                sb-assem::*backend-instruction-set-package*)
            (or (eq accessibility :external) (asm-inst-p symbol))
            ;; By default, retain any symbol with any attachments
-           (or (sb-kernel:symbol-info symbol)
+           (or (sb-kernel:symbol-%info symbol)
+               (sb-int:symbol-fdefn symbol)
                (and (boundp symbol) (not (keywordp symbol))))))))
    :verbose nil :print nil)
   (unintern 'sb-impl::shake-packages 'sb-impl)
@@ -475,6 +480,8 @@ Please check that all strings which were not recognizable to the compiler
             sum-delta-ext sum-delta-int
             (+ sum-delta-ext sum-delta-int))))
 
+(scan-format-control-strings)
+
 ;;; Lock internal packages
 #-(and sb-devel
        (not sb-devel-lock-packages))
