@@ -211,6 +211,13 @@
 (define-binop logior 2 orr)
 (define-binop logxor 2 eor)
 
+(define-vop (fast--/unsigned=>signed fast-unsigned-binop)
+  (:translate -)
+  (:results (r :scs (signed-reg)))
+  (:result-types signed-num)
+  (:generator 5
+    (inst sub r x y)))
+
 (define-binop logandc1 2 bic :swap t
   :constant-test bic-encode-immediate
   :constant-fixnum-test bic-fixnum-encode-immediate
@@ -915,33 +922,18 @@
   (:arg-types tagged-num tagged-num)
   (:note "inline fixnum comparison"))
 
-(define-vop (fast-conditional-c/fixnum fast-conditional/fixnum)
-  (:args (x :scs (any-reg)))
-  (:arg-types tagged-num (:constant (satisfies fixnum-abs-add-sub-immediate-p)))
-  (:info y))
-
 (define-vop (fast-conditional/signed fast-conditional)
   (:args (x :scs (signed-reg))
          (y :scs (signed-reg)))
   (:arg-types signed-num signed-num)
   (:note "inline (signed-byte 64) comparison"))
 
-(define-vop (fast-conditional-c/signed fast-conditional/signed)
-  (:args (x :scs (signed-reg)))
-  (:arg-types signed-num (:constant (satisfies abs-add-sub-immediate-p)))
-  (:info y))
-
 (define-vop (fast-conditional/unsigned fast-conditional)
   (:args (x :scs (unsigned-reg))
          (y :scs (unsigned-reg)))
   (:arg-types unsigned-num unsigned-num)
   (:note "inline (unsigned-byte 64) comparison"))
 
-(define-vop (fast-conditional-c/unsigned fast-conditional/unsigned)
-  (:args (x :scs (unsigned-reg)))
-  (:arg-types unsigned-num (:constant (satisfies abs-add-sub-immediate-p)))
-  (:info y))
-
 (defmacro define-conditional-vop (tran signed unsigned)
   `(progn
      ,@(loop for (suffix cost signed-p) in
@@ -988,6 +980,60 @@
 (define-conditional-vop > :gt :hi)
 (define-conditional-vop eql :eq :eq)
 
+(define-vop (<-unsigned-signed)
+  (:translate <)
+  (:args (unsigned :scs (unsigned-reg))
+         (signed :scs (signed-reg)))
+  (:arg-types unsigned-num signed-num)
+  (:conditional :lo)
+  (:policy :fast-safe)
+  (:generator 7
+    (inst cmp signed 0)
+    (inst ccmp unsigned signed :ge #b0010)))
+
+(define-vop (>-unsigned-signed <-unsigned-signed)
+  (:translate >)
+  (:conditional :hi))
+
+(define-vop (<-signed-unsigned)
+  (:translate <)
+  (:args (signed :scs (signed-reg))
+         (unsigned :scs (unsigned-reg)))
+  (:arg-types signed-num unsigned-num)
+  (:conditional :lo)
+  (:policy :fast-safe)
+  (:generator 7
+    (inst cmp signed 0)
+    (inst ccmp signed unsigned :ge #b0000)))
+
+(define-vop (>-signed-unsigned <-signed-unsigned)
+  (:translate >)
+  (:conditional :hi))
+
+(define-vop (eql-unsigned-signed)
+  (:translate eql)
+  (:args (unsigned :scs (unsigned-reg))
+         (signed :scs (signed-reg)))
+  (:arg-types unsigned-num signed-num)
+  (:conditional :eq)
+  (:policy :fast-safe)
+  (:generator 7
+    (inst cmp signed 0)
+    (inst ccmp unsigned signed :ge #b0000)))
+
+(define-vop (eql-signed-unsigned eql-unsigned-signed)
+  (:args (signed :scs (signed-reg))
+         (unsigned :scs (unsigned-reg)))
+  (:arg-types signed-num unsigned-num))
+
+(define-vop (eq-unsigned-signed eql-unsigned-signed)
+  (:variant-cost 6)
+  (:translate eq))
+
+(define-vop (eq-signed-unsigned eql-signed-unsigned)
+  (:variant-cost 6)
+  (:translate eq))
+
 (define-vop (generic-eql/fixnum fast-if-eql/fixnum)
   (:args (x :scs (any-reg descriptor-reg))
          (y :scs (any-reg)))
@@ -1038,27 +1084,18 @@
 (defun %logbitp (integer index)
   (logbitp index integer))
 
-(define-vop (fast-logbitp-c/fixnum fast-conditional-c/fixnum)
+(define-vop ()
   (:translate %logbitp)
+  (:policy :fast-safe)
+  (:args (x :scs (any-reg signed-reg unsigned-reg)))
+  (:info y)
+  (:arg-types (:or tagged-num signed-num unsigned-num) (:constant (mod #.n-word-bits)))
   (:conditional :ne)
-  (:arg-types tagged-num (:constant (mod #.n-word-bits)))
-  (:generator 4
-    (inst tst x (ash 1 (min (+ y n-fixnum-tag-bits)
-                            (1- n-word-bits))))))
-
-(define-vop (fast-logbitp-c/signed fast-conditional-c/signed)
-  (:translate %logbitp)
-  (:conditional :ne)
-  (:arg-types signed-num (:constant (mod #.n-word-bits)))
-  (:generator 5
-    (inst tst x (ash 1 y))))
-
-(define-vop (fast-logbitp-c/unsigned fast-conditional-c/unsigned)
-  (:translate %logbitp)
-  (:conditional :ne)
-  (:arg-types unsigned-num (:constant (mod #.n-word-bits)))
-  (:generator 5
-    (inst tst x (ash 1 y))))
+  (:generator 2
+    (inst tst x (ash 1 (min (if (sc-is x any-reg)
+                                (+ y n-fixnum-tag-bits)
+                                y)
+                         (1- n-word-bits))))))
 
 ;; Specialised mask-signed-field VOPs.
 (define-vop (mask-signed-field-word/c)
@@ -1328,3 +1365,451 @@
   (:translate sb-bignum:%ashl)
   (:generator 1
     (inst lsl result digit count)))
+
+(define-vop ()
+  (:translate sb-c::fixnum*)
+  (:args (x :scs (any-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types tagged-num tagged-num (:constant t))
+  (:info type)
+  (:temporary (:sc signed-reg) high)
+  (:results (r :scs (any-reg) :from :load))
+  (:result-types tagged-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::mul-overflow-error r high)))
+      (let (value)
+        (when (sc-is y immediate)
+          (load-immediate-word high (setf value (tn-value y)))
+          (setf y high))
+        (if (and value
+                 (plusp value)
+                 (= (logcount value) 1))
+            (inst lsl r x (1- (integer-length value)))
+            (inst mul r x y))
+        (inst smulh high x y))
+      (inst cmp high (asr r 63))
+      (inst b :ne error))))
+
+(define-vop ()
+  (:translate sb-c::signed*)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num (:constant t))
+  (:info type)
+  (:temporary (:sc signed-reg) high)
+  (:results (r :scs (signed-reg) :from :load))
+  (:result-types signed-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::mul-overflow-error r high)))
+      (let (value)
+        (when (sc-is y immediate)
+          (load-immediate-word high (setf value (tn-value y)))
+          (setf y high))
+        (if (and value
+                 (plusp value)
+                 (= (logcount value) 1))
+            (inst lsl r x (1- (integer-length value)))
+            (inst mul r x y))
+        (inst smulh high x y))
+      (inst cmp high (asr r 63))
+      (inst b :ne error))))
+
+(define-vop ()
+  (:translate sb-c::unsigned*)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num (:constant t))
+  (:info type)
+  (:temporary (:sc unsigned-reg) high)
+  (:results (r :scs (unsigned-reg) :from :load))
+  (:result-types unsigned-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::mul-overflow-error r high)))
+      (let ((value (and (sc-is y immediate)
+                        (tn-value y))))
+
+        (cond ((and value
+                    (plusp value)
+                    (= (logcount value) 1))
+               (let ((shift (1- (integer-length value))))
+                 (inst lsl r x shift)
+                 (inst lsr high x (- n-word-bits shift))))
+              (t
+               (when (sc-is y immediate)
+                 (load-immediate-word high value)
+                 (setf y high))
+               (inst mul r x y)
+               (inst umulh high x y))))
+      (inst cbnz high error))))
+
+(define-vop (*/signed=>integer)
+  (:translate *)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num)
+  (:temporary (:sc signed-reg) high low)
+  (:temporary (:sc signed-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 10
+    (let (value)
+      (when (sc-is y immediate)
+        (load-immediate-word high (setf value (tn-value y)))
+        (setf y high))
+      (if (and value
+               (plusp value)
+               (= (logcount value) 1))
+          (inst lsl low x (1- (integer-length value)))
+          (inst mul low x y))
+      (inst smulh high x y))
+    (inst mov header (bignum-header-for-length 2))
+    (inst cmp high (asr low 63))
+    (inst b :ne allocate)
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    #+bignum-assertions
+    (inst mov high 0)
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop (*/unsigned=>integer)
+  (:translate *)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num)
+  (:temporary (:sc unsigned-reg) high low)
+  (:temporary (:sc unsigned-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 12
+    (let ((value (and (sc-is y immediate)
+                        (tn-value y))))
+        (cond ((and value
+                    (plusp value)
+                    (= (logcount value) 1))
+               (let ((shift (1- (integer-length value))))
+                 (inst lsl low x shift)
+                 (inst lsr high x (- n-word-bits shift))))
+              (t
+               (when (sc-is y immediate)
+                 (load-immediate-word high value)
+                 (setf y high))
+               (inst mul low x y)
+               (inst umulh high x y))))
+    (inst mov header (bignum-header-for-length 3))
+    (inst tbnz high 63 allocate)
+    (inst mov header (bignum-header-for-length 2))
+    (inst cbnz high allocate)
+    (inst tbnz low 63 allocate)
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop (+/signed=>integer)
+  (:translate +)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num)
+  (:temporary (:sc signed-reg) high low)
+  (:temporary (:sc signed-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 10
+    (if (sc-is y immediate)
+        (let ((y (tn-value y)))
+          (if (minusp y)
+              (inst subs low x (add-sub-immediate (- y) high))
+              (inst adds low x (add-sub-immediate y high))))
+        (inst adds low x y))
+    (inst csetm high :cs)
+    (inst mov header (bignum-header-for-length 2))
+    (inst b :vs allocate)
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop (-/signed=>integer)
+  (:translate -)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num)
+  (:temporary (:sc signed-reg) high low)
+  (:temporary (:sc signed-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 10
+    (if (sc-is y immediate)
+        (let ((y (tn-value y)))
+          (if (minusp y)
+              (inst adds low x (add-sub-immediate (- y) high))
+              (inst subs low x (add-sub-immediate y high))))
+        (inst subs low x y))
+    (inst csetm high :cs)
+    (inst mov header (bignum-header-for-length 2))
+    (inst b :vs allocate)
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop (+/unsigned=>integer)
+  (:translate +)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num)
+  (:temporary (:sc unsigned-reg) high low)
+  (:temporary (:sc unsigned-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 10
+    (inst adds low x (if (sc-is y immediate)
+                         (add-sub-immediate (tn-value y) high)
+                         y))
+    (inst mov header (bignum-header-for-length 2))
+    (inst cset high :cs)
+    (inst b :cs allocate)
+    (inst b :mi allocate)
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop (-/unsigned=>integer)
+  (:translate -)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num)
+  (:temporary (:sc unsigned-reg) high low)
+  (:temporary (:sc unsigned-reg :from (:argument 2)) header)
+  (:temporary (:scs (non-descriptor-reg) :offset lr-offset) lr)
+  (:results (r :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 10
+    (inst subs low x (if (sc-is y immediate)
+                         (add-sub-immediate (tn-value y) high)
+                         y))
+    (inst mov header (bignum-header-for-length 2))
+    (inst csetm high :cc)
+    (inst b :cc negative)
+    (inst b :mi allocate)
+    (inst b positive)
+    negative
+    (inst b :pl allocate)
+    positive
+    (inst adds r low low)
+    (inst b :vc done)
+    (inst mov header (bignum-header-for-length 1))
+    allocate
+    (with-fixed-allocation
+        (r lr nil (+ 2 bignum-digits-offset))
+      (storew-pair header 0 low bignum-digits-offset tmp-tn)
+      (storew high tmp-tn 2))
+    DONE))
+
+(define-vop ()
+  (:translate sb-c::signed+)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num (:constant t))
+  (:info type)
+  (:results (r :scs (signed-reg)))
+  (:result-types signed-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (if (sc-is y immediate)
+          (let ((y (tn-value y)))
+            (if (minusp y)
+                (inst subs r x (add-sub-immediate (- y)))
+                (inst adds r x (add-sub-immediate y))))
+          (inst adds r x y))
+      (inst b :vs error))))
+
+(define-vop ()
+  (:translate sb-c::signed-)
+  (:args (x :scs (signed-reg))
+         (y :scs (signed-reg immediate)))
+  (:arg-types signed-num signed-num (:constant t))
+  (:info type)
+  (:results (r :scs (signed-reg)))
+  (:result-types signed-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (if (sc-is y immediate)
+          (let ((y (tn-value y)))
+            (if (minusp y)
+                (inst adds r x (add-sub-immediate (- y)))
+                (inst subs r x (add-sub-immediate y))))
+          (inst subs r x y))
+      (inst b :vs error))))
+
+(define-vop ()
+  (:translate sb-c::unsigned+)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num (:constant t))
+  (:info type)
+  (:results (r :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (when (sc-is y immediate)
+        (setf y (add-sub-immediate (tn-value y))))
+      (inst adds r x y)
+      (inst b :cs error))))
+
+(define-vop ()
+  (:translate sb-c::unsigned-)
+  (:args (x :scs (unsigned-reg))
+         (y :scs (unsigned-reg immediate)))
+  (:arg-types unsigned-num unsigned-num (:constant t))
+  (:info type)
+  (:results (r :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (when (sc-is y immediate)
+        (setf y (add-sub-immediate (tn-value y))))
+      (inst subs r x y)
+      (inst b :cc error))))
+
+(define-vop ()
+  (:translate sb-c::unsigned+signed)
+  (:args (x :scs (unsigned-reg) :to :result)
+         (y :scs (signed-reg immediate)))
+  (:arg-types unsigned-num signed-num (:constant t))
+  (:info type)
+  (:results (r :scs (unsigned-reg)))
+  (:temporary (:scs (unsigned-reg) :from (:argument 2)) temp)
+  (:result-types unsigned-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (if (sc-is y immediate)
+          (assemble ()
+            (let ((value (tn-value y)))
+              (setf y (add-sub-immediate (abs value)))
+              (cond ((plusp value)
+                     (inst adds r x y)
+                     (inst b :cs error))
+                    (t
+                     (inst subs r x y)
+                     (inst b :cc error)))))
+          (assemble ()
+            (when (sc-is y immediate)
+              (setf y (add-sub-immediate (tn-value y))))
+            (inst tbnz y 63 neg)
+            (inst adds r x y)
+            (inst b :cs error)
+            (inst b done)
+            neg
+            (inst neg temp y)
+            (inst subs r x temp)
+            (inst b :cc error)
+            done)))))
+
+(define-vop ()
+  (:translate sb-c::unsigned-signed)
+  (:args (x :scs (unsigned-reg) :to :result)
+         (y :scs (signed-reg immediate)))
+  (:arg-types unsigned-num signed-num (:constant t))
+  (:info type)
+  (:results (r :scs (unsigned-reg)))
+  (:temporary (:scs (unsigned-reg) :from (:argument 2)) temp)
+  (:result-types unsigned-num)
+  (:policy :fast-safe)
+  (:vop-var vop)
+  (:generator 2
+    (let* ((*location-context* (unless (eq type 'fixnum)
+                                 type))
+           (error (generate-error-code vop 'sb-kernel::add-sub-overflow-error r)))
+      (if (sc-is y immediate)
+          (assemble ()
+            (let ((value (tn-value y)))
+              (setf y (add-sub-immediate (abs value)))
+              (cond ((minusp value)
+                     (inst adds r x y)
+                     (inst b :cs error))
+                    (t
+                     (inst subs r x y)
+                     (inst b :cc error)))))
+          (assemble ()
+            (when (sc-is y immediate)
+              (setf y (add-sub-immediate (tn-value y))))
+            (inst tbz y 63 pos)
+            (inst neg temp y)
+            (inst adds r x temp)
+            (inst b :cs error)
+            (inst b done)
+            pos
+            (inst subs r x y)
+            (inst b :cc error)
+            done)))))
