@@ -270,8 +270,8 @@
                          (macrolet ((macro ()
                                       `((lambda (x)
                                           (declare (number x))
-                                          ',@ (loop repeat 10000
-                                                    for cons = (list 1) then (list cons)
+                                          ',@ (loop for cons = (list 1) then (list cons)
+                                                    repeat 10000
                                                     finally (return cons)))
                                         t)))
                            (macro)))
@@ -654,3 +654,42 @@
                       `(lambda (n c)
                          (make-array n :element-type 'bit :initial-contents (the string c)))
                       :allow-warnings 'warning))))
+
+(with-test (:name :make-array-initial-contents-constant-type-mismatch)
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda (n)
+                         (make-array n :element-type 'bit :initial-contents '(a b c)))
+                      :allow-warnings 'warning))))
+
+
+(with-test (:name :replace-constant-type-mismatch)
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda (x)
+                         (declare (string x))
+                         (replace x '(1 2 3)))
+                      :allow-warnings 'warning))))
+
+(with-test (:name :fill-type-mismatch)
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda (x)
+                         (declare (string x))
+                         (fill x 1))
+                      :allow-warnings 'warning))))
+
+(with-test (:name :vector-push-type-mismatch)
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda (x)
+                         (declare (string x))
+                         (vector-push 1 x))
+                      :allow-warnings 'warning)))
+  (assert (nth-value 2
+                     (checked-compile
+                      `(lambda (x)
+                         (declare (string x))
+                         (vector-push-extend 1 x))
+                      :allow-warnings 'warning))))
+
