@@ -15,6 +15,7 @@
   (load "compiler-test-util.lisp"))
 
 (declaim (optimize (debug 3) (speed 2) (space 1)))
+(declaim (muffle-conditions compiler-note))
 
 ;;; this would fail an AVER in NOTE-POTENTIAL-CIRCULARITY
 (defparameter *circular-2d-array* #1=#2A((a b) (#1# x)))
@@ -245,8 +246,8 @@
       (assert-canonical '(x y)) ; specifying all slots explicitly is still canonical
       (assert-canonical '(y x)))
     ;; specifying only one slot is not canonical
-    (assert (equal (let ((*foo-save-slots* '(x))) (sb-c::%make-load-form foo))
-                   '(sb-kernel::new-struct foo)))))
+    (let ((*foo-save-slots* '(x)))
+      (assert (not (sb-fasl:load-form-is-default-mlfss-p foo))))))
 
 ;; A huge constant vector. This took 9 seconds to compile (on a MacBook Pro)
 ;; prior to the optimization for using fops to dump.
@@ -335,7 +336,7 @@
 
 ;; Track the make-load-form FOPs as they fly by at load-time.
 (defvar *call-tracker* nil)
-(dolist (fop-name 'sb-fasl::(fop-allocate-instance fop-set-slot-values))
+(dolist (fop-name '(sb-fasl::fop-instance))
   (let* ((index (position fop-name sb-fasl::**fop-funs**
                           :key
                           (lambda (x) (and (functionp x) (sb-kernel:%fun-name x)))))
@@ -354,8 +355,7 @@
         (b p q r s)
         (c d e g))))
 
-(assert (= 14 (count 'sb-fasl::fop-allocate-instance *call-tracker*)))
-(assert (= 14 (count 'sb-fasl::fop-set-slot-values *call-tracker*)))
+(assert (= 14 (count 'sb-fasl::fop-instance *call-tracker*)))
 
 (with-test (:name :tree-with-parent-m-l-f-s-s)
   (verify-tree *y*))
@@ -485,6 +485,7 @@
     (assert (cadr z))))
 
 (macrolet ((expand ()
+             (declare (muffle-conditions compiler-note)) ; why is DECLAIM not enough?
              `(progn
                 ,@(loop for i from 0 below sb-vm:n-word-bits
                         collect
@@ -525,3 +526,20 @@
 
 (with-test (:name :coalesce-numeric-arrays)
   (assert (eq (use-numeric-vector-a) (use-numeric-vector-b))))
+
+(eval-when (:compile-toplevel :load-toplevel :execute)
+(defstruct person id)
+(defstruct (spy (:include person (id "   " :type (simple-string 2)))) gadgetry)
+(defmethod make-load-form ((self spy) &optional environment)
+  (declare (ignore environment))
+  (make-load-form-saving-slots self)))
+
+(defvar *agent* #.(let ((s (make-spy :id "86" :gadgetry '(shoe-phone))))
+                    (setf (person-id s) "max")
+                    s))
+
+;;; If lp#1969318 is fixed, then *AGENT* should not be reconstructed
+;;; from its load form, let alone dumped in the first place.
+;;; But thankfully we trap the read of the ID slot.
+(with-test (:name :illegal-typed-slot-value)
+  (assert-error (spy-id *agent*)))
