@@ -156,10 +156,10 @@
                  '(1 2 0))))
 
 (with-test (:name (print array *print-readably* :element-type))
-  (dolist (array (list (make-array '(1 0 1))
+  (dolist (array (list (make-array '(1 0 1) :initial-element 0)
                        (make-array 0 :element-type nil)
-                       (make-array 1 :element-type 'base-char)
-                       (make-array 1 :element-type 'character)))
+                       (make-array 1 :element-type 'base-char :initial-element #\nul)
+                       (make-array 1 :element-type 'character :initial-element #\nul)))
     (assert (multiple-value-bind (result error)
                 (read-from-string
                  (write-to-string array :readably t))
@@ -601,15 +601,15 @@
         (oops))
     (loop for f = most-positive-single-float then (/ f 2.0)
           while (> f 0.0)
-          do (loop repeat 10
-                   for fr = (random f)
+          do (loop for fr = (random f)
+                   repeat 10
                    do (unless (eql fr (read-from-string (prin1-to-string fr)))
                         (push fr oops)
                         (return))))
     (loop for f = most-negative-single-float then (/ f 2.0)
           while (< f -0.0)
-          do (loop repeat 10
-                   for fr = (- (random (- f)))
+          do (loop for fr = (- (random (- f)))
+                   repeat 10
                    do (unless (eql fr (read-from-string (prin1-to-string fr)))
                         (push fr oops)
                         (return))))
@@ -625,16 +625,16 @@
     ;; FIXME skipping denormalized floats due to bug 793774.
     (loop for f = most-positive-double-float then (/ f 2d0)
           while (> f 0d0)
-          do (loop repeat 10
-                   for fr = (random f)
+          do (loop for fr = (random f)
+                   repeat 10
                    do (unless (float-denormalized-p fr)
                         (unless (eql fr (read-from-string (prin1-to-string fr)))
                           (push fr oops)
                           (return)))))
     (loop for f = most-negative-double-float then (/ f 2d0)
           while (< f -0d0)
-          do (loop repeat 10
-                   for fr = (- (random (- f)))
+          do (loop for fr = (- (random (- f)))
+                   repeat 10
                    do (unless (float-denormalized-p fr)
                         (unless (eql fr (read-from-string (prin1-to-string fr)))
                           (push fr oops)
@@ -872,6 +872,11 @@ there"))))
     (let ((str (write-to-string x :pretty nil)))
       (assert (search "#<SB-KERNEL:FUNCALLABLE-INSTANCE {" str)))))
 
+(with-test (:name :functionless-funcallable-instance)
+  (let ((x (sb-pcl::%make-standard-funcallable-instance #() #xdead)))
+    (assert (search "#<SB-PCL::STANDARD-FUNCALLABLE-INSTANCE"
+                    (write-to-string x :pretty nil))))) ; should not crash
+
 (with-test (:name :bug-885320)
   (let* ((form `(format nil "~E"))
          (fun (compile nil `(lambda (x) (,@form x)))))
