@@ -13,6 +13,21 @@
 
 (cl:in-package :cl-user)
 
+(defun bin-pwd-ignoring-result ()
+  (let ((initially-open-fds (directory "/proc/self/fd/*" :resolve-symlinks nil)))
+    (sb-ext:run-program "/usr/bin/pwd" nil :input :stream :output :stream :wait nil)
+    (length initially-open-fds)))
+
+(with-test (:name (run-program :autoclose-streams)
+            :skipped-on (:or :sbcl ;; not reliable enough
+                             (:not :linux)))
+  (let ((n-initially-open-fds (bin-pwd-ignoring-result)))
+    (gc)
+    (sb-sys:scrub-control-stack) ; Make sure we're not referencing the #<process>
+    (gc) ; now nothing should reference the streams
+    (assert (= (length (directory "/proc/self/fd/*" :resolve-symlinks nil))
+               n-initially-open-fds))))
+
 ;; In addition to definitions lower down the impurity we're avoiding
 ;; is the sigchld handler that RUN-PROGRAM sets up, which interfers
 ;; with the manual unix process control done by the test framework
@@ -166,14 +181,13 @@
 ;;; agressive about flushing them. So, here's another test using :PTY.
 
 #-win32
-(with-test (:name :is-/bin/ed-installed?)
-  (assert (probe-file "/bin/ed")))
+(unless (probe-file "/bin/ed") (push :no-bin-ed-installed *features*))
 
 #-win32
 (progn
   (defparameter *tmpfile* (scratch-file-name))
 
-  (with-test (:name (run-program :/bin/ed))
+  (with-test (:name (run-program :/bin/ed) :skipped-on :no-bin-ed-installed)
     (with-open-file (f *tmpfile*
                        :direction :output
                        :if-exists :supersede)
@@ -269,7 +283,8 @@
 ;; We can't check for the signal itself since run-program.c resets the
 ;; forked process' signal mask to defaults. But the default is `stop'
 ;; of which we can be notified asynchronously by providing a status hook.
-(with-test (:name (run-program :inherit-stdin) :fails-on :win32)
+(with-test (:name (run-program :inherit-stdin) :fails-on :win32
+                  :skipped-on :no-bin-ed-installed)
   (let (stopped)
     (flet ((status-hook (proc)
              (case (process-status proc)
@@ -417,7 +432,7 @@
 
 (with-test (:name (run-program :malloc-deadlock)
             :broken-on :sb-safepoint
-            :skipped-on (or (not :sb-thread) :win32))
+            :skipped-on (or :ubsan (not :sb-thread) :win32))
   (let* (stop
          (delay-between-gc
           (or #+freebsd
